#include <sys/types.h>
#include <system_def.h>
#include <types.h>
#include <cpu_def.h>
#include <mx1_reg.h>
#include <irq_def.h>

extern long __excpt_pass_struct_base[MAX_EXCEPTIONS*2];
extern long __excpt_cook_handler[MAX_EXCEPTIONS];

long __excpt_stack_start[MAX_EXCEPTIONS];
long __excpt_vector_target[MAX_EXCEPTIONS];

int set_irq_priority(int irqnum, int level)
{
  if(irqnum>64)
    return -1;

  unsigned long flags;
  int levshift;
  unsigned long levmask;

  save_and_cli(flags);

  if(level<IRQ_PRIORITY_LEVEL_FIQ){
    if(irqnum<32)
      _reg_AITC_INTTYPEL &= ~(1 << ((irqnum)&0x1f));
    else
      _reg_AITC_INTTYPEH &= ~(1 << ((irqnum - 32)&0x1f));

    levshift = (irqnum%8)*4;
    levmask = 0xf << levshift;
    _reg_AITC_NIPRIORITY(irqnum/8) &= ~levmask;
    _reg_AITC_NIPRIORITY(irqnum/8) |= levmask & (level << levshift);
  }else{
    if(irqnum<32)
      _reg_AITC_INTTYPEL |= (1 << ((irqnum)&0x1f));
    else
      _reg_AITC_INTTYPEH |= (1 << ((irqnum - 32)&0x1f));
  } 
  restore_flags(flags);
  return 0;
}

unsigned long
set_cpu_exception_handler(int cpu_exception, long target_addr)
{
  unsigned long flags;
  unsigned long old_value;
  unsigned long mode=cpu_exception&CPU_EXPTION_TYPE_MASK;
  
  cpu_exception&=~CPU_EXPTION_TYPE_MASK;

  if(cpu_exception>=MAX_EXCEPTIONS)
    return 0;

  save_and_cli(flags);

  if(!(mode&CPU_EXPTION_TYPE_RAW)) {
    __excpt_vector_target[cpu_exception]=target_addr;
    target_addr=__excpt_cook_handler[cpu_exception];
  }

  /* e59ff018        ldr     pc, [pc, #24] */
  __excpt_pass_struct_base[cpu_exception]=0xe59ff000+(MAX_EXCEPTIONS*4-8);
  old_value=__excpt_pass_struct_base[cpu_exception+MAX_EXCEPTIONS];
  __excpt_pass_struct_base[cpu_exception+MAX_EXCEPTIONS]=target_addr;

  restore_flags(flags);

  return old_value;
}

unsigned long
set_cpu_exception_stack(int cpu_exception, unsigned long new_stack)
{
  long old_stack;
  if(cpu_exception>=MAX_EXCEPTIONS)
    return 0;

  old_stack = __excpt_stack_start[cpu_exception];
  __excpt_stack_start[cpu_exception] = new_stack;
  return old_stack;
}
