#ifndef _ARM_IRQ_DEF_H
#define _ARM_IRQ_DEF_H

#include <sys/types.h>
#include <system_def.h>
#include <types.h>
#include <cpu_def.h>
#include <mx1_reg.h>

typedef enum {
  ARM_EXCEPTION_RESET      = 0,
  ARM_EXCEPTION_UNDEF      = 1,
  ARM_EXCEPTION_SWI        = 2,
  ARM_EXCEPTION_PREF_ABORT = 3,
  ARM_EXCEPTION_DATA_ABORT = 4,
  ARM_EXCEPTION_RESERVED   = 5,
  ARM_EXCEPTION_IRQ        = 6,
  ARM_EXCEPTION_FIQ        = 7,
  MAX_EXCEPTIONS           = 8
} CPU_EXCEPTION_TYPES;


/* Enables interrupt vector. */
static inline void 
enable_irq(int irqnum)
{
	/* Enable interrupt */
	_reg_AITC_INTENNUM = irqnum;
	return;
}

/* Disables spi_mx1 interrupts */
static inline void 
disable_irq(int irqnum)
{
	/* disable all various TX/RX interrupts */
	_reg_AITC_INTDISNUM = irqnum;
	return;
}

/* Tests to see if spi_mx1 interrupts are enabled, and returns non-0 if so.
 * If interrupt is not enabled, returns 0.
 */
static inline  int
is_enabled_irq(int irqnum)
{
	if(irqnum<32)
		return _reg_AITC_INTENABLEL & (1 << ((irqnum)&0x1f));
	else
		return _reg_AITC_INTENABLEH & (1 << ((irqnum - 32)&0x1f));
}

static inline  int
is_asserted_irq(int irqnum)
{
	if(irqnum<32)
		return _reg_AITC_INTSRCL & (1 << ((irqnum)&0x1f));
	else
		return _reg_AITC_INTSRCH & (1 << ((irqnum - 32)&0x1f));
}

static inline  int
is_pending_irq(int irqnum)
{
	if(irqnum<32)
		return _reg_AITC_NIPNDL & (1 << ((irqnum)&0x1f));
	else
		return _reg_AITC_NIPNDH & (1 << ((irqnum - 32)&0x1f));
}

static inline  int
is_pending_fiq(int irqnum)
{
	if(irqnum<32)
		return _reg_AITC_FIPNDL & (1 << ((irqnum)&0x1f));
	else
		return _reg_AITC_FIPNDH & (1 << ((irqnum - 32)&0x1f));
}

static inline int
get_irq_mask_level(void)
{
	return _reg_AITC_NIMASK&0xf;
}

static inline void
set_irq_mask_level(int level)
{
	_reg_AITC_NIMASK=level&0xf;
}

#define IRQ_PRIORITY_LEVEL_FIQ 16

int set_irq_priority(int irqnum, int level);

unsigned long
set_cpu_exception_handler(int cpu_exception, long target_addr);
#define CPU_EXPTION_TYPE_MASK   0xF0000000
#define CPU_EXPTION_TYPE_RAW    0x10000000
#define CPU_EXPTION_TYPE_COOKED 0x20000000

unsigned long
set_cpu_exception_stack(int cpu_exception, unsigned long new_stack);

#endif /*_ARM_IRQ_DEF_H*/
