/*******************************************************************
  Components for embedded applications builded for
  laboratory and medical instruments firmware  
 
  sgm_lcd.h - low level support for Small Graphic Module
              LCD display (240x64x1) based on TOSHIBA T6963
	      controller
 
  Copyright (C) 2001 by Pavel Pisa pisa@cmp.felk.cvut.cz
            (C) 2002 by PiKRON Ltd. http://www.pikron.com

 *******************************************************************/

#ifndef _SGM_LCD_H_
#define _SGM_LCD_H_

#include <types.h>

/* Status flags in LCD_STAT */
#define SGM_LCD_RF_CMD	0x01	/* ready for command */
#define SGM_LCD_RF_DATA	0x02	/* ready for data */
#define SGM_LCD_RF_ARD	0x04	/* ready for auto mode read */
#define SGM_LCD_RF_AWR	0x08	/* ready for auto mode write */
#define SGM_LCD_CHK_CAP	0x20	/* check capability */
#define SGM_LCD_ERROR	0x40	/* peek/copy error */
#define SGM_LCD_BLINK	0x80	/* 1=normal, 0=display off */

/* Control codes for LCD_CMD */
#define SGM_LCD_S_CP	0x21	/* cursor pointer set D1=X, D2=Y */
#define SGM_LCD_S_OFS	0x22	/* offset register D1=data, D2=0 */
#define SGM_LCD_S_ADP	0x24	/* address register D1=low, D2=high */
#define SGM_LCD_C_THADR	0x40	/* text home address set D1=low, D2=high */
#define SGM_LCD_C_TCOL	0x41	/* text area set D1=columns, D2=0 */
#define SGM_LCD_C_GHADR	0x42	/* graphics home address set D1=low, D2=high */
#define SGM_LCD_C_GCOL	0x43	/* graphics area set D1=columns, D2=0 */
#define SGM_LCD_M_OR	0x80	/* OR mode */
#define SGM_LCD_M_EXOR	0x81	/* EXOR mode */
#define SGM_LCD_M_AND	0x83	/* AND mode */
#define SGM_LCD_M_TXTA	0x84	/* text attribute mode */
#define SGM_LCD_M_ICG	0x80	/* internal CG-ROM mode */
#define SGM_LCD_M_ECG	0x88	/* external CG-ROM mode */
#define SGM_LCD_D_OFF	0x90	/* display off */
#define SGM_LCD_D_CON	0x92	/* cursor on */
#define SGM_LCD_D_CONB	0x93	/* cursor on and blinking */
#define SGM_LCD_D_TXT	0x94	/* text on */
#define SGM_LCD_D_GRAPH	0x98	/* graphic on */
#define SGM_LCD_D_BOTH	0x9C	/* graphic and text on */
#define SGM_LCD_CPAT	0xA0	/* +number of of lines */
#define SGM_LCD_A_WR	0xB0	/* data auto write set */
#define SGM_LCD_A_RD	0xB1	/* data auto read set */
#define SGM_LCD_A_RES	0xB2	/* auto reset */
#define SGM_LCD_WR_INC	0xC0	/* data write and ADP increment */
#define SGM_LCD_RD_INC	0xC1	/* data read and ADP increment */
#define SGM_LCD_WR_DEC	0xC2	/* data write and ADP decrement */
#define SGM_LCD_RD_DEC	0xC3	/* data read and ADP decrement */
#define SGM_LCD_WR_NV	0xC4	/* data write and ADP nonvariable */
#define SGM_LCD_RD_NV	0xC5	/* data read and ADP nonvariable */
#define SGM_LCD_PEEK	0xE0	/* screen peek */
#define SGM_LCD_COPY	0xE8	/* screen copy */
#define SGM_LCD_BITSET	0xF0	/* +bit number to set */
#define SGM_LCD_BITRES	0xF8	/* +bit number to reset */

/* state variables */

__u16 sgm_lcd_text_hadr;	/* internal address of text area start */
__u16 sgm_lcd_graph_cols;	/* number of graphics columns per line */
__u16 sgm_lcd_text_cols;	/* number of text columns per line */
short sgm_lcd_init_ok;		/* information about initialization state */

/* low level routines */

/* wait until ready for command state */
void sgm_lcd_rf_cmd(void);

/* wait until ready for mask (alternatives SGM_LCD_RF_CMD,_DATA,_ARD,_AWR) */
void sgm_lcd_rf_x(int mask);

/* command with no parameters */
void sgm_lcd_cmd_d0(int cmd);

/* command with one parameter */
void sgm_lcd_cmd_d1(int cmd, int data1);

/* command with two parameters */
void sgm_lcd_cmd_d2(int cmd, int data1, int data2);

/* command returning byte value */
int sgm_lcd_cmd_rd(int cmd);

/* command with internal address parameter */
void sgm_lcd_cmd_adr(int cmd, int data);

/* data write */
void sgm_lcd_wrbyte(short val);

/* data block write at actual address */
void sgm_lcd_wrblock(unsigned char *block, int dup);

/* character write */
void sgm_lcd_wrchr(short chr);

/* zero terminated string write */
void sgm_lcd_wrstr(char *s);

/* fill dup bytes of internal memory by chr */
void sgm_lcd_fill_chr(int dup, char chr);

/* set next character write position in text buffer */
void sgm_lcd_gotoxy(short x, short y);

/* initialize cotroller */
int sgm_lcd_init(void);

#endif /*  _SGM_LCD_H_ */
