#include <cpu_def.h>
#include <irq_def.h>
#include <mx1_def.h>
#include <mx1_reg.h>
#include <types.h>
#include <stdarg.h>
#include <char_que.h>
#include <string.h>
#include <periph/mx1_boardctrl.h>

#define WITH_SLEEP_WAKE		/*enable wake-up from keyboard*/

struct sleep_gpio_regs_save {
	volatile uint32_t DDIR;
	volatile uint32_t OCR1;
	volatile uint32_t OCR2;
	volatile uint32_t DR;
	volatile uint32_t GIUS;
	volatile uint32_t IMR;
	volatile uint32_t GPR;
	volatile uint32_t PUEN;
};

#if defined (WITH_SLEEP_WAKE) || defined(WITH_IRQ_CATCH)
void mx1_setup_irq_on_power_on(void)
{
  int i;
  /*IRQ_PRIORITY_LEVEL_FIQ*/
  set_irq_priority(MX1_GPIO_INT_PORTD, 7);
  enable_irq(MX1_GPIO_INT_PORTD);
  set_irq_mask_level(0);

  _reg_PTD_DDIR&=~(1<<POWER_ON_BUTTON_ON_PTD_b);
  _reg_PTD_PUEN|=(1<<POWER_ON_BUTTON_ON_PTD_b);
 #if POWER_ON_BUTTON_ON_PTD_b>=16
  _reg_PTD_ICR2&=~(3<<((POWER_ON_BUTTON_ON_PTD_b-16)*2));
  _reg_PTD_ICR2|=(1<<((POWER_ON_BUTTON_ON_PTD_b-16)*2));
 #else
  _reg_PTD_ICR1&=~(3<<((POWER_ON_BUTTON_ON_PTD_b)*2));
  _reg_PTD_ICR1|=(1<<((POWER_ON_BUTTON_ON_PTD_b)*2));
 #endif
  for(i=0;i<0x1000;i++){
    _reg_PTD_ISR |= (1<<POWER_ON_BUTTON_ON_PTD_b);
  }
  _reg_PTD_IMR |= (1<<POWER_ON_BUTTON_ON_PTD_b);
  
  _reg_AITC_INTCNTL|=MX1_AITC_INTCNTL_NIAD_m;
  /*_reg_AITC_INTCNTL|=MX1_AITC_INTCNTL_FIAD_m;*/
}
#endif /*WITH_SLEEP_WAKE || WITH_IRQ_CATCH*/

static inline
void mx1_sleep_sethw4sleep(void)
{
  _reg_SDRAMC_SDCTL0&=0xFFFFEFFF;
   sl_udelay(5000);
  _reg_PTA_GIUS=0x00FFFFFE; 
  _reg_PTA_OCR1=0xFFFFFFFF; 
  _reg_PTA_OCR2=0xFFFFFFFF; 
  _reg_PTA_DDIR=0xFF00701D; 
  _reg_PTA_PUEN=0x00FF8FE2; 
//  _reg_PTA_DR=0x00238000; 

  _reg_PTB_GIUS=0xFFFFFFFF; 
  _reg_PTB_OCR1=0xFFFFFFFF; 
  _reg_PTB_OCR2=0xFFFFFFFF; 
  _reg_PTB_DDIR=0x30000000; 
//  _reg_PTB_PUEN=0xFCFFC0FF; 
  _reg_PTB_PUEN=0xCCFFDFFF; 
  _reg_PTB_DR=0x00200004; /* With main power supply off */
//  _reg_PTB_DR=0x20200004; /* Without main power supply off */

  _reg_PTC_GIUS=0xFFFFFFFF; 
  _reg_PTC_OCR1=0xFFFFFFFF; 
  _reg_PTC_OCR2=0xFFFFFFFF; 
  _reg_PTC_DDIR=0x00000000; 
  _reg_PTC_PUEN=0xFFFC1E8F; 
  _reg_PTC_DR=0x01001000; 

  _reg_PTD_GIUS=0xFFFFFFFF; 
  _reg_PTD_OCR1=0xFFFFFFFF; 
  _reg_PTD_OCR2=0xFFFFFFFF;
  _reg_PTD_DDIR=0x7FFFFA40;
  _reg_PTD_PUEN=0x800005BF; 
  _reg_PTD_DR=0x00000000; 

  mx1_sdram_refresh_off();
}

static inline
void mx1_sleep_sethw4wake(void)
{
  unsigned long l;
  /* Enable main and MSP430 power */
  _reg_PTB_DR|=1<<29;
  
  for(l=50000;l;l--){
    __asm__ __volatile__ ("":::"memory");
  }

  mx1_sdram_init();
}

void mx1_sleep()
{
  unsigned long flags;
  int i;
  struct sleep_gpio_regs_save ptx_save_buffer[4];
  struct sleep_gpio_regs_save *ptx_save;
  volatile struct mx1_gpio_regs *ptx_regs;
  uint32_t save_intenableh, save_intenablel, save_intcntl;
  unsigned long save_level;
  
  fiq_save_and_cli(flags);

  save_intcntl=_reg_AITC_INTCNTL;
  _reg_AITC_INTCNTL=0;

  save_intenableh=_reg_AITC_INTENABLEH;
  save_intenablel=_reg_AITC_INTENABLEL;
  save_level=get_irq_mask_level();

  for(i=0;i<4;i++){
    ptx_regs=&_reg_PTx(i);
    ptx_save=&ptx_save_buffer[i];

    ptx_save->DDIR  = ptx_regs->DDIR;
    ptx_save->OCR1  = ptx_regs->OCR1;
    ptx_save->OCR2  = ptx_regs->OCR2;
    ptx_save->DR    = ptx_regs->DR;
    ptx_save->GIUS  = ptx_regs->GIUS;
    ptx_save->IMR   = ptx_regs->IMR;
    ptx_save->GPR   = ptx_regs->GPR;
    ptx_save->PUEN  = ptx_regs->PUEN;
    
    ptx_regs->IMR   = 0;
  }
  
  mx1_sleep_sethw4sleep();

#ifdef WITH_SLEEP_WAKE
  mx1_setup_irq_on_power_on();
#endif /*WITH_SLEEP_WAKE*/

#if 1
  /* Schedule i.MX processor to switch oscillator off after SD_CNT controlled delay */
  _reg_PLL_CSCR&=0xFFFFFFFC;

  /* Enter power down mode */
  __asm__ __volatile__(
   "  mov r0,#0\n\t"
   "  mcr p15,0,r1,c7,c0,4\n\t":::"memory","r0","r1","r2","r3","r4","r5"
  );
#else
  /*while(!(_reg_PTD_ISR&(1<<POWER_ON_BUTTON_ON_PTD_b)));*/
  /*while(!is_asserted_irq(MX1_GPIO_INT_PORTD));*/
  while(!is_pending_irq(MX1_GPIO_INT_PORTD));
#endif

  mx1_sleep_sethw4wake();

  for(i=0;i<4;i++){
    ptx_regs=&_reg_PTx(i);
    ptx_save=&ptx_save_buffer[i];

    ptx_regs->PUEN  |= ptx_save->PUEN;
    ptx_regs->DDIR  &= ptx_save->DDIR;

    ptx_regs->OCR1  = ptx_save->OCR1;
    ptx_regs->OCR2  = ptx_save->OCR2;
    ptx_regs->DR    = ptx_save->DR;
    ptx_regs->GIUS  = ptx_save->GIUS;
    ptx_regs->IMR   = ptx_save->IMR;
    ptx_regs->GPR   = ptx_save->GPR;
    ptx_regs->PUEN  = ptx_save->PUEN;
    ptx_regs->DDIR  = ptx_save->DDIR;
  }

#ifdef WITH_SLEEP_WAKE
  _reg_PTD_ISR = (1<<POWER_ON_BUTTON_ON_PTD_b);
#endif /*WITH_SLEEP_WAKE*/

  _reg_AITC_INTENABLEH=save_intenableh;
  _reg_AITC_INTENABLEL=save_intenablel;

  _reg_AITC_INTCNTL=save_intcntl;

  set_irq_mask_level(save_level);

  restore_flags(flags);

}
