#include <mx1_reg.h>
#include <types.h>
#include <stdarg.h>
#include <char_que.h>
#include <periph/mx1_uart_ll.h>

inline volatile
struct mx1_uartx_regs *mx1_uartx_chan2base(int uartch)
{
  switch(uartch){
    case 1:
      return &_reg_UART1;
    case 2:
      return &_reg_UART2;
    case 3:
      return &_reg_UART3;
    default:
      return 0;
  }
}

int mx1_uartx_setmode(int uartch, int baud, int mode, int flowc)
{
  volatile struct mx1_uartx_regs *puart;
  unsigned long mask;
  switch(uartch){
    case 1:
      /* GPIO port C */
      mask=(1<<12)|(1<<11);	/* RxD and TxD */
      if(0){
        mask|=(1<<10)|(1<<9);	/* RTS and CTS */
      }
      _reg_PTC_GIUS&= ~mask ;	/* GIUS_C */
      _reg_PTC_GPR&= ~mask ;	/* GPR_C */
      break;
    case 2:
      /* GPIO port B */
      mask=(1<<31)|(1<<30);	/* RxD and TxD */
      if(0){
        mask|=(1<<29)|(1<<28);	/* RTS and CTS */
      }
      _reg_PTB_GIUS&= ~mask ;	/* GIUS_B */
      _reg_PTB_GPR&= ~mask ;	/* GPR_B */
      if(0){			/* DSR, RI, DCD and DTR */
        mask=(1<<10)|(1<<9)|(1<<8)|(1<<7);
        _reg_PTD_GIUS&= ~mask ;	/* GIUS_D */
	_reg_PTD_GIUS |= (1<<7);
        _reg_PTD_GPR |= mask ;	/* GPR_D */
      }
      break;
    case 3:
      {
	unsigned long iconfa2_clr;
	unsigned long ocr12_set;
	unsigned long ocr12_clr;
	/* GPIO port C */
	mask=(1<<31)|(1<<30);	/* RxD and TxD */
	iconfa2_clr=(1<<31)|(1<<30);
	ocr12_set=(1<<28);
	ocr12_clr=(1<<29);
	if(0){
          mask|=(1<<29)|(1<<28);	/* RTS and CTS */
	  iconfa2_clr|=(1<<27)|(1<<26);
	  ocr12_set|=(1<<24);
	  ocr12_clr|=(1<<25);
	}
	if(0){			/* DSR, RI, DCD and DTR */
          mask=(1<<25)|(1<<24)|(1<<27)|(1<<26);
	  iconfa2_clr|=(1<21)|(1<20);
	  ocr12_set|=(1<<18)|(1<<16)|(1<<22);
	  ocr12_clr|=(1<<19)|(1<<17)|(1<<23);
	}
	_reg_PTC_ICONFA2 &=~ iconfa2_clr;
	_reg_PTC_OCR1 |=  ocr12_set;
	_reg_PTC_OCR1 &=~ ocr12_clr;
	_reg_PTC_OCR2 |=  ocr12_set;
	_reg_PTC_OCR2 &=~ ocr12_clr;
	_reg_PTC_GIUS |= mask ;	/* GIUS_C */
      }
      break;
    default:
      return MX1_UART_LL_ERROR;
  }
  puart=mx1_uartx_chan2base(uartch);
  if(!puart)
      return MX1_UART_LL_ERROR;

  /* CLOCK: */
  _reg_PLL_PCDR |= 0x000000f;  /* PCDR  PCLKDIV1=3(/4) => 24MHz */
  _reg_PLL_PCDR &= 0x0000003 | ~0x000000f;  /*0x000B003B*/
  /* CONTROL: */
  puart->UCR1=0x00000005;
  puart->UCR2=0x00004026;
  puart->UCR3=0x00000000;
  puart->UCR4=0x00008000;
  
  puart->UFCR=0x00008201;
  /* BAUD: */
  puart->UBIR=255;
  puart->UBMR=9999;
  
  return 0;
}


int mx1_uartx_sendready(int uartch)
{
  volatile struct mx1_uartx_regs *puart;
  puart=mx1_uartx_chan2base(uartch);
  if(!puart)
      return MX1_UART_LL_ERROR;
  return (puart->USR1 & MX1_UX_USR1_TRDY_m)?1:0;
}

int mx1_uartx_recready(int uartch)
{
  volatile struct mx1_uartx_regs *puart;
  puart=mx1_uartx_chan2base(uartch);
  if(!puart)
      return MX1_UART_LL_ERROR;
  return (puart->USR2 & MX1_UX_USR2_RDR_m)?1:0;

}

int mx1_uartx_senddone(int uartch)
{
  volatile struct mx1_uartx_regs *puart;
  puart=mx1_uartx_chan2base(uartch);
  if(!puart)
      return MX1_UART_LL_ERROR;
  return (puart->USR2 & MX1_UX_USR2_TXDC_m)?1:0;
}

int mx1_uartx_sendch(int uartch,unsigned char ch)
{
  volatile struct mx1_uartx_regs *puart;
  puart=mx1_uartx_chan2base(uartch);
  if(!puart)
      return MX1_UART_LL_ERROR;
  
  if(!(puart->USR1 & MX1_UX_USR1_TRDY_m))
    return MX1_UART_LL_EAGAIN;
  puart->UTXD[0]=ch;  
  return ch;
}

int mx1_uartx_recch(int uartch)
{
  unsigned int ch;
  volatile struct mx1_uartx_regs *puart;
  puart=mx1_uartx_chan2base(uartch);
  if(!puart)
      return MX1_UART_LL_ERROR;
  
  if(!(puart->USR2 & MX1_UX_USR2_RDR_m))
    return MX1_UART_LL_EAGAIN;
  ch=puart->URXD[0];
  return ch & MX1_UX_URXD_RX_DATA_m;
}
