#include <types.h>
#include <stdarg.h>
#include <char_que.h>
#include <system_def.h>
#include <malloc.h>
#include <string.h>
#include <periph/mx1_uart_ll.h>
#include <periph/mx1_uartcon.h>

int mx1_uartcon_uartch;
int mx1_uartcon_buffered_out;

#define QUE_SIZE 0x10000
char_que_t mx1_uartcon_que_out = {
  .buf_beg=NULL,
  .buf_end=NULL,
  .ip=NULL,
  .op=NULL
};

int char_que_alloc_buf(char_que_t *q, size_t size)
{
  uint8_t *buf;
  if((buf=q->buf_beg)!=NULL){
    q->buf_beg = q->buf_end = q->ip = q->op = NULL;
    free(buf);
  }
  buf=malloc(size);
  if(!buf)
    return -1;
  q->buf_beg = q->ip = q->op = buf;
  q->buf_end = buf+=size;
  return 0;
}

int mx1_uartcon_test_in(void)
{
  return mx1_uartx_recready(mx1_uartcon_uartch);
}

int mx1_uartcon_test_out(void)
{
  return mx1_uartx_sendready(mx1_uartcon_uartch);
}

/* reads character throught CPU32Bug */
int mx1_uartcon_getchar(void)
{ 
  int ch;
  do{
    ch=mx1_uartx_recch(mx1_uartcon_uartch);
  } while(ch==MX1_UART_LL_EAGAIN);
  return ch;
}

int mx1_uartcon_putchar(int ch)
{
  int res;

  if(!mx1_uartcon_buffered_out){
    do{
      res=mx1_uartx_sendch(mx1_uartcon_uartch,ch);
    } while(res==MX1_UART_LL_EAGAIN);
  }else{
    res=char_que_put(&mx1_uartcon_que_out,ch);
  }
  return res;
}

int mx1_uartcon_process(void)
{
  int res=0;
  if(mx1_uartcon_buffered_out && mx1_uartx_sendready(mx1_uartcon_uartch)){
    int ch;
    if((ch=char_que_get(&mx1_uartcon_que_out))>=0){
      res=mx1_uartx_sendch(mx1_uartcon_uartch,ch);
      if(res<0)
        return res;
      res=1;
    }
  }
  return res;
}

int mx1_uartcon_write(int file, char * ptr, int len)
{
  int cnt;
  unsigned char ch;
  for(cnt=0;cnt<len;cnt++,ptr++){
    ch=*ptr;
    if(ch==0xa)
      mx1_uartcon_putchar(0xd);
    mx1_uartcon_putchar(ch);
  }
  return cnt;
}

int mx1_uartcon_up(int uartch, int mode)
{
  if(uartch>=0){
    mx1_uartcon_uartch=uartch;
  }
  if(mode & MX1_UARTCON_BUFFERED){
    if(char_que_alloc_buf(&mx1_uartcon_que_out, QUE_SIZE)>=0)
      mx1_uartcon_buffered_out=1;
    else
      mx1_uartcon_buffered_out=0;
  }else{
    mx1_uartcon_buffered_out=0;
  }
  system_stub_ops.write=mx1_uartcon_write;
  return 0;
}
