#include <usb/usb_spec.h>
#include <usb/usbdebug.h>
#include <usb/usb_loader.h>
#include <usb_mx1/mx1_usb_ll.h>
#include <mx1_reg.h>
#include <flashlib/flashlib.h>

#include "mx1_usb_defs.h"

#define USB_DEBUG_OFF_AFTER 5

int usb_mx1_simple_debcnt=0;

int usb_mx1_simple_init( void)
{
  usb_mx1_simple_debcnt=0;
  usb_debug_print( DEBUG_LEVEL_LOW, ("USB Simple Init Called\n"));
  return mx_usb_init_ll();
}


int usb_mx1_simple_ctrl_reply(void *data, int len, int zlps_fl)
{
  int i;
  int res;

  /*_reg_USBD_EP(0).STAT |= MX1_EP_STAT_FLUSH_m;
  while (_reg_USBD_EP(0).STAT & MX1_EP_STAT_FLUSH_m);*/

  while(len>0){
    if(len<8){
      zlps_fl=0;
      i=len;
    }else{
      i=8;
    }

    while(__mfld2val(MX1_EP_STAT_BYTE_COUNT_m,_reg_USBD_EP(0).STAT)>32-i){
      ; // busywait for free space
    }

    while((res=mx_usb_ep_write(0, data, i))!=i){
      if(res<0){
	usb_debug_print( DEBUG_LEVEL_LOW, ("Reply write failed\n"));
	return -1;
      }
      usb_debug_print( DEBUG_LEVEL_MEDIUM, ("USB WR Busy wait\n"));
      res&=MX_USB_RW_EP_LEN_m;
      i-=res;
      data=(char*)data+res;
    }
    data=(char*)data+i;
    len-=i;
  }

  if(zlps_fl)
    _reg_USBD_EP(0).STAT |= MX1_EP_STAT_ZLPS_m;    /* next one is a zero length packet */
  return 0;
}

int usb_mx1_simple_ctrl_read(void *data, int len)
{
  int res;
  int busy_fl=0;
  uint32_t istat, stat;

  while((res=mx_usb_ep_read(0, data, len | MX_USB_RW_EP_NOFR_m))!=len){
    istat = _reg_USBD_EP(0).INTR_STAT;
    if(istat & MX1_EP_INTR_MDEVREQ_m){
      usb_debug_print( DEBUG_LEVEL_MEDIUM, ("usb_mx1_simple_ctrl_read MDEVREQ detect\n"));
      _reg_USBD_EP(0).INTR_STAT = MX1_EP_INTR_MDEVREQ_m;
    }
    /*stat = _reg_USBD_EP(0).STAT;
    if(istat & MX1_EP_STAT_SIP_m){
      usb_debug_print( DEBUG_LEVEL_MEDIUM, ("usb_mx1_simple_ctrl_read SIP detect\n"));
    }*/
    
    if(res<0){
      if(busy_fl) usb_debug_print( DEBUG_LEVEL_MEDIUM, ("\n"));
      usb_debug_print( DEBUG_LEVEL_LOW, ("Read control data failed\n"));
      return -1;
    }
    if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
      usb_debug_print( DEBUG_LEVEL_MEDIUM, (busy_fl++?".":"USB RD Busy wait "));
    }
    res&=MX_USB_RW_EP_LEN_m;
    len-=res;
    data=(char*)data+res;
  }
  if(busy_fl) usb_debug_print( DEBUG_LEVEL_MEDIUM, ("\n"));

  return 0;
}

#define MX1_FLASH_BUFF 1024

int usb_mx1_simple_prog_flash(unsigned long addr, int len)
{
  uint8_t buff[MX1_FLASH_BUFF];
  int blen=MX1_FLASH_BUFF;
  int i;
  int res;
  uint8_t *p;
  
  while(len>0){
    i=len>blen?blen:len;
    if((res=mx_usb_ep_read(0, buff, i))!=i) {
      if(res<0){
	usb_debug_print( DEBUG_LEVEL_LOW, ("Read control data failed\n"));
	return -1;
      }
      if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
        usb_mx1_simple_debcnt++;
        usb_debug_print( DEBUG_LEVEL_MEDIUM, ("USB Flash short\n"));
      }
      res&=MX_USB_RW_EP_LEN_m;
    }

    p=buff;
    while(res>0) {
      if((addr<flash_alg_default->start)||
         (addr>=flash_alg_default->start+flash_alg_default->size)){
        usb_debug_print( DEBUG_LEVEL_MEDIUM, ("Addr not in flash\n"));
        return 0;
      }
      
      i=flash_alg_default->prog(flash_alg_default, addr, p, res);
      if(i<0){
        usb_debug_print( DEBUG_LEVEL_MEDIUM, ("ERROR flash_prog returned %d\n",i));
        return -1;
      }
      p+=i;
      addr=addr+i;
      len-=i;
      res-=i;
    }
  }

  return 0;
}


int usb_mx1_simple_get_set_memory(USB_DEVICE_REQUEST *dreq)
{
  unsigned long addr;
  unsigned len;
  int res=0;
  addr = (dreq->wValue & 0xffff) | (((unsigned long)dreq->wIndex&0xffff)<<16);
  len = dreq->wLength;
  if (( dreq->bmRequestType & USB_DATA_DIR_MASK) == USB_DATA_DIR_FROM_HOST) {
    // read from HOST ???
    switch( dreq->bRequest & USB_VENDOR_TARGET_MASK) {
      case USB_VENDOR_TARGET_RAM:
        if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
	  usb_mx1_simple_debcnt++;
          usb_debug_print( DEBUG_LEVEL_LOW, ("USB SET MEMORY, addr=0x%lx, len=%d\n", (long)addr, len));
	} else {
          usb_debug_print( DEBUG_LEVEL_LOW, ("W"));
	}
	res=usb_mx1_simple_ctrl_read((void *)addr, len);
	break;
      case USB_VENDOR_TARGET_FLASH:
        if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
	  usb_mx1_simple_debcnt++;
          usb_debug_print( DEBUG_LEVEL_LOW, ("USB SET FLASH, addr=0x%lx, len=%d\n", (long)addr, len));
	} else {
          usb_debug_print( DEBUG_LEVEL_LOW, ("F"));
	}
	res=usb_mx1_simple_prog_flash(addr, len);
	break;
      default:
        usb_debug_print( DEBUG_LEVEL_LOW, ("USB SET MEMORY, bad target %d\n", dreq->bRequest & USB_VENDOR_TARGET_MASK));
        return -1;
    }
    //if ( len) usb_set_control_endfnc( udev, usb_ack_setup);
    //else usb_ack_setup(&udev->ep0);
    return 1;
  } else {
    switch( dreq->bRequest & USB_VENDOR_TARGET_MASK) {
      case USB_VENDOR_TARGET_RAM:
      case USB_VENDOR_TARGET_FLASH:
        if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
	  usb_mx1_simple_debcnt++;
          usb_debug_print( DEBUG_LEVEL_LOW, ("USB GET MEMORY, addr=0x%lx, len=%d\n", (long)addr, len));
	} else {
          usb_debug_print( DEBUG_LEVEL_LOW, ("R"));
	}
	res=usb_mx1_simple_ctrl_reply((void *)addr, len, 0);
	break;
      default:
        usb_debug_print( DEBUG_LEVEL_LOW, ("USB GET MEMORY, bad target %d\n", dreq->bRequest & USB_VENDOR_TARGET_MASK));
        return -1;
    }
  }
  return res;
}

typedef int (local_call_t)(void);

int usb_mx1_simple_go_addr(USB_DEVICE_REQUEST *dreq)
{
  unsigned int go_delay=0x10000;
  unsigned long addr;
  addr = (dreq->wValue & 0xffff) | (((unsigned long)dreq->wIndex&0xffff)<<16);


  _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m;
  _reg_USBD_INTR_MASK &= ~(MX1_USBD_INTR_SOF_m);
  
  usb_debug_print( DEBUG_LEVEL_LOW, ("USB_VENDOR_GOTO, addr=0x%lx\n", (long)addr));

  while(go_delay--){
    if(__mfld2val(MX1_EP_STAT_BYTE_COUNT_m,_reg_USBD_EP(0).STAT)){
      usb_debug_print(DEBUG_LEVEL_MEDIUM, ("Flushing data:\n"));
      while(!(_reg_USBD_EP(0).FSTAT&MX1_EP_FSTAT_EMPTY_m)){
	unsigned x;
	x = (((uint8_t*)&(_reg_USBD_EP(0).FDAT))[0]);
	usb_debug_print(DEBUG_LEVEL_MEDIUM, (" %02x",x));
	if(go_delay<1000)
          go_delay+=0x100;
      }
    }
    _reg_USBD_EP(0).INTR_STAT=~0;
    _reg_USBD_INTR_MASK &= ~(MX1_USBD_INTR_SOF_m);
  }

  _reg_USBD_CTRL = 0;
  
  sl_prepare4appstart();

  ((local_call_t*)addr)();
  
  return 0;
}

int usb_mx1_simple_mass_erase(unsigned mode, unsigned addr)
{
  int res;

  usb_debug_print(DEBUG_LEVEL_MEDIUM, ( "MASSERASE mode=0x%X addr=0x%X\n", mode, addr));

  /* StrataFlash memory */
  if(!addr)
    addr=0x10000000;

  res=flash_alg_default->erase(flash_alg_default, addr, -1);

  if(res>=0){
    usb_debug_print(DEBUG_LEVEL_MEDIUM, ( "ERASE OK\n"));
  }else{
    usb_debug_print(DEBUG_LEVEL_MEDIUM, ( "ERASE ERROR %d\n",res));
  }

  return (res<0)?res:0;
}

int usb_mx1_simple_erase_region(unsigned long addr, int len)
{
  int res;

  if((addr<flash_alg_default->start)||
    (addr+len>flash_alg_default->start+flash_alg_default->size)){
    usb_debug_print( DEBUG_LEVEL_MEDIUM, ("Erase addr not in flash\n"));
    return 0;
  }

  if(res>=0){
    usb_debug_print(DEBUG_LEVEL_MEDIUM, ( "ERASE Reg 0x%08lx size 0x%08lx OK\n", addr, len));
  }else{
    usb_debug_print(DEBUG_LEVEL_MEDIUM, ( "ERASE Reg 0x%08lx size 0x%08lx ERROR %d\n", addr, len,res));
  }

  res=flash_alg_default->erase(flash_alg_default, addr, len);

  return (res<0)?res:0;
}

/**
 * usb_mx1_simple_run
 * function reads interrupt register and sets event flags
 * function returns 1 if there is some new event.
 * function returns 0 if there isn't new event but all is OK
 * function returns -1 if there is any error
*/
int usb_mx1_simple_run( void) {
    unsigned int ep_int;
    unsigned int dev_int;
    int ret = 0, i;
    int forceread;
    USB_DEVICE_REQUEST dreq_buf;
    USB_DEVICE_REQUEST *dreq=&dreq_buf;

    dev_int = _reg_USBD_INTR_STAT;
    _reg_USBD_INTR_STAT = dev_int;
   
    if (dev_int & MX1_USBD_INTR_SOF_m){
      _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m;	// signal CMD_OVER
      dev_int &= ~MX1_USBD_INTR_SOF_m;
    }
    dev_int &= ~MX1_USBD_INTR_MSOF_m;			// We do not care for this

    if (dev_int) {
      if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
        usb_debug_print( DEBUG_LEVEL_LOW, ("USB INTR=0x%04X\n", dev_int));
      }
    } else {
      //usb_debug_print( DEBUG_LEVEL_VERBOSE, ("USB No Int\n"));
    }
    
    if ( dev_int & MX1_USBD_INTR_RESET_START_m) {	// Bus reset reached
      usb_debug_print( DEBUG_LEVEL_LOW, ("USB Reset requested\n"));
      mx_usb_reset();
      ret = 1;
    } else {
      if ( dev_int & MX1_USBD_INTR_SUSP_m) {		// Suspend flag changed
        ret = 1;
      }

      if ( dev_int & MX1_USBD_INTR_WAKEUP_m) {		// Suspend flag changed
        ret = 1;
      }
            
      forceread=__mfld2val(MX1_EP_STAT_BYTE_COUNT_m,_reg_USBD_EP(0).STAT)>=8;
      ep_int=_reg_USBD_EP(0).INTR_STAT;
      if(ep_int)
        forceread=0;
      if(ep_int & MX1_EP_INTR_MDEVREQ_m){
	usb_debug_print( DEBUG_LEVEL_MEDIUM, ("usb_mx1_simple_ctrl_read MDEVREQ detect\n"));
	_reg_USBD_EP(0).INTR_STAT = MX1_EP_INTR_MDEVREQ_m;
      }
      _reg_USBD_EP(0).INTR_STAT=ep_int & (~MX1_EP_INTR_DEVREQ_m);
      if((ep_int & MX1_EP_INTR_DEVREQ_m) || forceread) { // Ep0 SetupPacketReceived
	void *rep_ptr=0;
	int rep_len=-1;
	int zlps_fl=0;
	int res;
	
	if(forceread){
          usb_debug_print( DEBUG_LEVEL_LOW, ("EP0 Forced read\n"));
          usb_mx1_simple_debcnt=0;
	}
        if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
          usb_debug_print( DEBUG_LEVEL_LOW, ("EP0 Drq INTR=0x%04X\n", ep_int));
	}
	res=mx_usb_ep_read(0, dreq, 8);
	if(res>=0){
	  int taken = res&MX_USB_RW_EP_LEN_m;
	  if((taken<8) && (taken>0)){
            usb_debug_print( DEBUG_LEVEL_LOW,( "USB DEVRQ short read %08x\n",res));
	    res=mx_usb_ep_read(0, ((uint8_t*)dreq)+taken, 8-taken);
	    if(res>0)
	      res+=taken;
	  }
	}

        _reg_USBD_EP(0).INTR_STAT=MX1_EP_INTR_DEVREQ_m;

	if((res<0) || ((res&MX_USB_RW_EP_LEN_m)!=8)){
          usb_debug_print( DEBUG_LEVEL_LOW,( "USB DEVRQ mx_usb_ep_read failed %08x\n",res));
	  _reg_USBD_EP(0).STAT|=MX1_EP_STAT_FLUSH_m;
	  return -1;
	}
	
        if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
          usb_debug_print( DEBUG_LEVEL_MEDIUM,( "SePa:x%02X,x%02X,x%04X,x%04X,x%04X\n", dreq->bmRequestType, dreq->bRequest, dreq->wValue, dreq->wIndex, dreq->wLength));
	}

        switch(dreq->bmRequestType & USB_REQUEST_TYPE_MASK) {
	  case USB_STANDARD_REQUEST:
           
	    if(((dreq->bmRequestType&USB_RECIPIENT)==USB_RECIPIENT_DEVICE)&&
	       ((dreq->bmRequestType&USB_DATA_DIR_MASK)==USB_DATA_DIR_TO_HOST)&&
	       (dreq->bRequest==USB_REQUEST_GET_DESCRIPTOR)) {
	      i = (dreq->wValue >> 8) & 0xff; /* MSB part of wValue */
	      usb_debug_print( DEBUG_LEVEL_VERBOSE, ( "GetDesc - %s desc.\n", /*(unsigned int)*/ usb_debug_get_std_descriptor(i)));

              switch(i) {
		case USB_DESCRIPTOR_TYPE_DEVICE:
        	  rep_ptr = (unsigned char *)&DeviceDescription;
        	  rep_len = sizeof( USB_DEVICE_DESCRIPTOR);
        	  break;
		case USB_DESCRIPTOR_TYPE_CONFIGURATION:
        	  rep_ptr = (unsigned char *)&ConfigDescription;
        	  rep_len = CONFIG_DESCRIPTOR_LENGTH;
        	  break;
		case USB_DESCRIPTOR_TYPE_INTERFACE:
        	  rep_ptr = (unsigned char *)&ConfigDescription.interface;
        	  rep_len = sizeof( USB_INTERFACE_DESCRIPTOR);
        	  break;
		case USB_DESCRIPTOR_TYPE_STRING:
		  zlps_fl=1;
        	  i = dreq->wValue & 0xff; /* LSB part of wValue */
        	  usb_debug_print( DEBUG_LEVEL_VERBOSE, ("Get string descriptor indx=0x%02x\n", i));
        	  if ( i < CNT_STRINGS) {
        	    rep_ptr = (unsigned char *) StringDescriptors[ i];
        	    rep_len = *(unsigned char*)rep_ptr;
        	    /*usb_debug_print(0,("indx=0x%02x ptr=%p rep_len=%d : '%c'\n", i, rep_ptr, rep_len, rep_ptr[2]));*/
        	  } else {
	            _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m | MX1_USBD_CTRL_CMD_ERROR_m;
        	    /*return USB_COMPLETE_FAIL;*/
        	  }
        	  break;
		default:
	           usb_debug_print( DEBUG_LEVEL_LOW, ("Unknown standard request 0x%02x\n",i));
	          _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m | MX1_USBD_CTRL_CMD_ERROR_m;
        	  /*return USB_COMPLETE_FAIL;*/
		  rep_len = 0;
	      }
	      if ( dreq->wLength < rep_len) rep_len = dreq->wLength;
	      usb_debug_print( DEBUG_LEVEL_VERBOSE, ("Reply %d bytes\n", rep_len));

	      if(rep_len>=0){
	        usb_mx1_simple_ctrl_reply(rep_ptr, rep_len, zlps_fl);
	      }

	    }
	  
	    break;
          case USB_VENDOR_REQUEST:
	    switch(dreq->bRequest) {
	      
	      case USB_VENDOR_GET_SET_MEMORY | USB_VENDOR_TARGET_RAM:
	      case USB_VENDOR_GET_SET_MEMORY | USB_VENDOR_TARGET_FLASH:
		if(usb_mx1_simple_get_set_memory(dreq)<0) {
  	          _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m | MX1_USBD_CTRL_CMD_ERROR_m;
		}
		break;
		
	      case USB_VENDOR_GOTO:
		usb_mx1_simple_go_addr(dreq);
		break;

	      case USB_VENDOR_MASS_ERASE:
		usb_mx1_simple_mass_erase(dreq->wIndex, dreq->wValue);
		break;
	      case USB_VENDOR_ERASE_MEMORY:  /* erase memory for 1 Byte */
		if(usb_mx1_simple_erase_region(dreq->wValue, dreq->wIndex)<0){
  	          _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m | MX1_USBD_CTRL_CMD_ERROR_m;
		}
		break;
	      case USB_VENDOR_ERASE_1KB_MEMORY:  /* erase memory for 1 KB */
		if(usb_mx1_simple_erase_region((uint32_t)dreq->wValue<<10 | flash_alg_default->start, 
					    (uint32_t)dreq->wIndex<<10)<0){
  	          _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m | MX1_USBD_CTRL_CMD_ERROR_m;
		}
		break;
	      default:
                usb_debug_print( DEBUG_LEVEL_LOW, ("Unknown vendor request 0x%02x\n",dreq->bRequest));
	        _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m | MX1_USBD_CTRL_CMD_ERROR_m;
        	/*return USB_COMPLETE_FAIL;*/
	    }
	    break;
	  default:
            usb_debug_print( DEBUG_LEVEL_LOW, ("Unknown request type 0x%02x\n",dreq->bmRequestType));
	    _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m | MX1_USBD_CTRL_CMD_ERROR_m;
	
	}

	//_reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m;

        /* turn on SOF interrupt to signal CMD_OVER bit */
        _reg_USBD_INTR_MASK &= ~(MX1_USBD_INTR_SOF_m);

      } else {
        if(ep_int & MX1_EP_INTR_EOF_m) {
          if(usb_mx1_simple_debcnt<USB_DEBUG_OFF_AFTER) {
            usb_debug_print( DEBUG_LEVEL_LOW, ("EP0 INTR=0x%04X\n", ep_int));
	  }
        }
      }

      /*      
      for( i=0; i<udev->cntep; i++) {
        ep_int=_reg_USBD_EP(udev->ep[i].epnum).INTR_STAT;
        _reg_USBD_EP(udev->ep[i].epnum).INTR_STAT=ep_int;
        if (ep_int & MX1_EP_INTR_EOF_m) {
          udev->ep_events |= 1<<i;
          ret = 1;
        }
      }
      */

    }
    return ret;
  }
